enum MergeState<B: Batch> {
Vacant,
Single(Option<B>),
Double(MergeVariant<B>),
}Expand description
Describes the state of a layer.
A layer can be empty, contain a single batch, or contain a pair of batches that are in the process of merging into a batch for the next layer.
Variants
Vacant
An empty layer, containing no updates.
Single(Option<B>)
A layer containing a single batch.
The None variant is used to represent a structurally empty batch present
to ensure the progress of maintenance work.
Double(MergeVariant<B>)
A layer containing two batches, in the process of merging.
Implementations
Sourceimpl<B: Batch<Time: Eq>> MergeState<B>
impl<B: Batch<Time: Eq>> MergeState<B>
Sourcefn complete(&mut self) -> Option<(B, Option<(B, B)>)>
fn complete(&mut self) -> Option<(B, Option<(B, B)>)>
Immediately complete any merge.
The result is either a batch, if there is a non-trivial batch to return
or None if there is no meaningful batch to return. This does not distinguish
between Vacant entries and structurally empty batches, which should be done
with the is_complete() method.
There is the additional option of input batches.
Sourcefn is_complete(&mut self) -> bool
fn is_complete(&mut self) -> bool
True iff the layer is a complete merge, ready for extraction.
Sourcefn work(&mut self, fuel: &mut isize)
fn work(&mut self, fuel: &mut isize)
Performs a bounded amount of work towards a merge.
If the merge completes, the resulting batch is returned. If a batch is returned, it is the obligation of the caller to correctly install the result.
Sourcefn begin_merge(
batch1: Option<B>,
batch2: Option<B>,
compaction_frontier: AntichainRef<'_, B::Time>,
) -> MergeState<B>
fn begin_merge( batch1: Option<B>, batch2: Option<B>, compaction_frontier: AntichainRef<'_, B::Time>, ) -> MergeState<B>
Initiates the merge of an “old” batch with a “new” batch.
The upper frontier of the old batch should match the lower frontier of the new batch, with the resulting batch describing their composed interval, from the lower frontier of the old batch to the upper frontier of the new batch.
Either batch may be None which corresponds to a structurally
empty batch whose upper and lower froniers are equal. This
option exists purely for bookkeeping purposes, and no computation
is performed to merge the two batches.